==> openejb-core-4.7.2-sources.jar: org/apache/openejb/core/ivm/naming/LazyObjectReference.java <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.openejb.core.ivm.naming;

import javax.naming.NamingException;
import java.util.concurrent.Callable;

public class LazyObjectReference<T> extends Reference {
    private Callable<T> creator;
    private volatile T instance;

    public LazyObjectReference(final Callable<T> creator) {
        this.creator = creator;
    }

    @Override
    public Object getObject() throws NamingException {
        if (instance == null) {
            synchronized (this) {
                if (instance == null) {
                    try {
                        instance = creator.call();
                    } catch (final Exception e) {
                        throw new NamingException(e.getMessage());
                    }
                }
            }
        }
        return instance;
    }

    public boolean isInitialized() {
        return instance != null;
    }
}

==> openejb-core-5.0.0-20150513.040812-291-sources.jar: org/apache/openejb/core/ivm/naming/LazyObjectReference.java <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.openejb.core.ivm.naming;

import javax.naming.NamingException;
import java.util.concurrent.Callable;

public class LazyObjectReference<T> extends Reference {
    private final Callable<T> creator;
    private volatile T instance;

    public LazyObjectReference(final Callable<T> creator) {
        this.creator = creator;
    }

    @Override
    public Object getObject() throws NamingException {
        if (instance == null) {
            synchronized (this) {
                if (instance == null) {
                    try {
                        instance = creator.call();
                    } catch (final Exception e) {
                        throw new NamingException(e.getMessage());
                    }
                }
            }
        }
        return instance;
    }

    public boolean isInitialized() {
        return instance != null;
    }
}
